// ============================================================================
// $Id: tokenizer.hh,v 1.7 1998/03/20 15:56:06 chris Exp $
//
// This is part of Realm Of Magic
//
// Usage: Splitting a string's substrings that are seperated by special chars
//
// All rights reserved. See license.doc for complete information.
//
// Copyright (C) 1993-1996 by the Hackerz of the Realm Of Magic
// Realm Of Magic is based on CircleMUD, Copyright (C) 1993.
// CircleMUD is based on DikuMUD, Copyright (C) 1990, 1991.
// ============================================================================

#ifndef __ROM_SYS__TOKENIZER_HH__
#define __ROM_SYS__TOKENIZER_HH__

#include <string>
#include <list>
#include <algorithm>

/** The Tokenizer Class
  It splits a string, that is made up of substrings and
  seperatorchars for those substrings into it's components.
  These components are accessible through the operator[]
 */

class tokenizer {
public:

/** The Constructor
  Creates an empty tokenizer...note that empty tokenizers can not be
  accessed and 0 is returned when someone tries to do so.
 */
  tokenizer();
  
/** The Constructor
  You have to give the string, and the seperating string.
 */
  tokenizer(const string, const string);
  
/** The Copy Constructor
  Creates a token-object from an already existing one. Because
  of the way, the list of substrings is copied, the parameter
  can't be const.
 */
  tokenizer(tokenizer&);

/** Cut-Constructor
  Improvement suggested by Criscal: the value gives the starting position
  for the to-be-tokenized string.
 */
  tokenizer(const string, const string, string::size_type);
      
/** The operator[]
  It gives you access to the substrings, the tokens, where
  [0] is the first one.
 */
  const char *operator[] (int);

/** Returns the number of tokens the string was split into.
 */
  const int size() { return tokenamount; }

/** Returns, whether a tokenizer is empty or not.
 */
  const bool empty() { return emptystate; }
 
/** Empties the tokenizer, and makes it ready for re-write.
 */
  void clear();

/** Gives tokenstring and seperatorchar to an empty tokenizer and
  tokenizes it. If tokenizer is already full, false is returned.
 */
  bool tokenize(string, string);

private:
  string tokenstring;
  list<string> tokens;
  string seperatorstring;
  bool tokenizesuccess;
  int tokenamount;
  bool emptystate;
};

#endif  
